"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _cryptoJs = _interopRequireDefault(require("crypto-js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var DEFAULT_ENCRYPTION_TYPE = 'AES';

var encode = function () {
  var _ref2 = _asyncToGenerator(function* (_ref) {
    var {
      secret,
      key = secret,
      token = {},
      maxAge,
      encryption = DEFAULT_ENCRYPTION_TYPE
    } = _ref;

    if (maxAge) {
      if (token.iat) {
        delete token.iat;
      }

      if (token.exp) {
        delete token.exp;
      }
    }

    var signedToken = _jsonwebtoken.default.sign(token, secret, {
      expiresIn: maxAge
    });

    switch (encryption) {
      case false:
        return signedToken;

      case 'AES':
        return _cryptoJs.default.AES.encrypt(signedToken, key).toString();

      default:
        throw new Error('Unsupported value for `encryption` passed to JWT encode()', encryption);
    }
  });

  return function encode(_x) {
    return _ref2.apply(this, arguments);
  };
}();

var decode = function () {
  var _ref4 = _asyncToGenerator(function* (_ref3) {
    var {
      secret,
      key = secret,
      token,
      maxAge,
      encryption = DEFAULT_ENCRYPTION_TYPE
    } = _ref3;
    if (!token) return null;
    var tokenToVerify;

    switch (encryption) {
      case false:
        tokenToVerify = token;
        break;

      case 'AES':
        {
          var decryptedBytes = _cryptoJs.default.AES.decrypt(token, key);

          var decryptedToken = decryptedBytes.toString(_cryptoJs.default.enc.Utf8);
          tokenToVerify = decryptedToken;
          break;
        }

      default:
        throw new Error('Unsupported value for `encryption` passed to JWT decode()', encryption);
    }

    return _jsonwebtoken.default.verify(tokenToVerify, secret, {
      maxAge
    });
  });

  return function decode(_x2) {
    return _ref4.apply(this, arguments);
  };
}();

var getToken = function () {
  var _ref6 = _asyncToGenerator(function* (_ref5) {
    var {
      req,
      secret,
      key = secret,
      maxAge,
      encryption,
      secureCookie,
      cookieName
    } = _ref5;
    if (!req) throw new Error('Must pass `req` to JWT getToken()');

    if (!cookieName) {
      if (typeof secureCookie === 'undefined') {
        secureCookie = !(!process.env.NEXTAUTH_URL || process.env.NEXTAUTH_URL.startsWith('http://'));
      }

      cookieName = secureCookie ? '__Secure-next-auth.session-token' : 'next-auth.session-token';
    }

    var token = req.cookies[cookieName];

    if (!token && req.headers.authorization && req.headers.authorization.split(' ')[0] === 'Bearer') {
      var urlEncodedToken = req.headers.authorization.split(' ')[1];
      token = decodeURIComponent(urlEncodedToken);
    }

    try {
      return yield decode({
        secret,
        key,
        token,
        maxAge,
        encryption
      });
    } catch (error) {
      return null;
    }
  });

  return function getToken(_x3) {
    return _ref6.apply(this, arguments);
  };
}();

var _default = {
  encode,
  decode,
  getToken
};
exports.default = _default;